// ignore_for_file: public_member_api_docs, sort_constructors_first
import 'dart:async';
import 'dart:io';

import 'package:codenic_alarm/src/models/alarm_info.dart';
import 'package:codenic_alarm/src/platforms/codenic_alarm_android.dart';
import 'package:codenic_alarm/src/platforms/codenic_alarm_unsupported_platform.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

/// {@template CodenicAlarm}
/// An interface for scheduling alarms on Android.
/// {@endtemplate}
abstract class CodenicAlarm extends PlatformInterface {
  /// {@macro CodenicAlarm}
  CodenicAlarm() : super(token: _token);

  static final Object _token = Object();

  static CodenicAlarm _instance = Platform.isAndroid
      ? CodenicAlarmAndroid()
      : CodenicAlarmUnsupportedPlatform();

  /// The default instance of [CodenicAlarm] to use.
  static CodenicAlarm get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [CodenicAlarm] when
  /// they register themselves.
  static set instance(CodenicAlarm instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  /// Initializes the plugin.
  ///
  /// This must be called at the start of the app.
  ///
  /// Returns `true` if the plugin was successfully initialized or has already
  /// been initialized. Otherwise, `false` is returned..
  FutureOr<bool> initialize();

  /// Schedules an alarm to be executed once in the future after the specified
  /// [delay].
  FutureOr<bool> schedule({
    required Duration delay,
    required AlarmInfo alarmInfo,
  });

  /// Schedules an alarm to be executed once in the future at the specified
  /// [schedule].
  FutureOr<bool> scheduleAt({
    required DateTime schedule,
    required AlarmInfo alarmInfo,
  });

  /// Cancels the alarm with the specified [id].
  FutureOr<void> cancel({required int id});

  /// Cancels all scheduled alarms.
  FutureOr<void> cancelAll();

  /// Dismisses the currently triggered or active alarm.
  ///
  /// If there are no active alarms, this method will do nothing.
  FutureOr<void> dismissAlarm();

  /// Returns the currently active alarm, if any.
  ///
  /// This can be used to display the active alarm in the UI or check if an
  /// alarm is currently active.
  FutureOr<AlarmInfo?> activeAlarm();
}
