/// {@template AlarmInfo}
/// A model containing information about an alarm.
/// {@endtemplate}
class AlarmInfo {
  /// {@macro AlarmInfo}
  const AlarmInfo({
    required this.id,
    required this.title,
    required this.content,
  });

  /// Creates an [AlarmInfo] from a JSON object.
  factory AlarmInfo.fromJson(Map<String, dynamic> json) {
    return AlarmInfo(
      id: json['id'] as int,
      title: json['title'] as String,
      content: json['content'] as String,
    );
  }

  /// The unique identifier of the alarm.
  final int id;

  /// The title of the alarm.
  final String title;

  /// The content of the alarm.
  final String content;

  /// Converts the [AlarmInfo] to a JSON object.
  Map<String, dynamic> toJson() {
    return <String, dynamic>{
      'id': id,
      'title': title,
      'content': content,
    };
  }
}
